]> git.sesse.net Git - movit/blobdiff - effect.h
Make Input an abstract base class, and move the current functionality into FlatInput...
[movit] / effect.h
index 397d3643975545b94fca94aa5eed58aff8ee4c85..d14176b2b1f4dc5f74f56388766cdfb2d5d14362 100644 (file)
--- a/effect.h
+++ b/effect.h
@@ -60,7 +60,9 @@ public:
        // set of RGB primaries; you would currently not get YCbCr
        // or something similar).
        //
-       // Again, most effects will want this.
+       // Again, most effects will want this, but you can set it to false
+       // if you process each channel independently, equally _and_
+       // in a linear fashion.
        virtual bool needs_srgb_primaries() const { return true; }
 
        // Whether this effect expects its input to come directly from
@@ -94,11 +96,16 @@ public:
        // needs mipmaps, you will also get them).
        virtual bool needs_mipmaps() const { return false; }
 
+       // How many inputs this effect will take (a fixed number).
+       // If you have only one input, it will be called INPUT() in GLSL;
+       // if you have several, they will be INPUT1(), INPUT2(), and so on.
+       virtual unsigned num_inputs() const { return 1; }
+
        // Requests that this effect adds itself to the given effect chain.
        // For most effects, the default will be fine, but for effects that
        // consist of multiple passes, it is often useful to replace this
        // with something that adds completely different things to the chain.
-       virtual void add_self_to_effect_chain(EffectChain *graph, Effect *input);
+       virtual void add_self_to_effect_chain(EffectChain *graph, const std::vector<Effect *> &inputs);
 
        // Outputs one GLSL uniform declaration for each registered parameter
        // (see below), with the right prefix prepended to each uniform name.
@@ -108,17 +115,19 @@ public:
        // Returns the GLSL fragment shader string for this effect.
        virtual std::string output_fragment_shader() = 0;
 
// Set all uniforms the shader needs in the current GL context.
// The default implementation sets one uniform per registered parameter.
        //
        // <sampler_num> is the first free texture sampler. If you want to use
        // textures, you can bind a texture to GL_TEXTURE0 + <sampler_num>,
        // and then increment the number (so that the next effect in the chain
        // will use a different sampler).
-       //
-       // NOTE: Currently this is also abused a bit to set other GL state
-       // the effect might need.
-       virtual void set_uniforms(GLuint glsl_program_num, const std::string& prefix, unsigned *sampler_num);
+       virtual void set_gl_state(GLuint glsl_program_num, const std::string& prefix, unsigned *sampler_num);
+
+       // If you set any special OpenGL state in set_gl_state(), you can clear it
+       // after rendering here. The default implementation does nothing.
+       virtual void clear_gl_state();
 
        // Set a parameter; intended to be called from user code.
        // Neither of these take ownership of the pointer.
@@ -108,17 +115,19 @@ public:
        virtual std::string output_fragment_shader() = 0;
 
-       // Set all uniforms the shader needs in the current GL context.
-       // The default implementation sets one uniform per registered parameter.
+       // Set all OpenGL state that this effect needs before rendering.
+       // The default implementation sets one uniform per registered parameter,
+       // but no other state.
        //
        // <sampler_num> is the first free texture sampler. If you want to use